from .custom_api import CustomApi
from .hikkalls import HikkaLls
from .stream_type import StreamType
from .sync import idle

__all__ = (
    'idle',
    'CustomApi',
    'HikkaLls',
    'StreamType',
)



import asyncio
from typing import Union

from ...exceptions import NodeJSNotRunning
from ...exceptions import NoMtProtoClientSet
from ...exceptions import NotInGroupCallError
from ...mtproto import BridgedClient
from ...scaffold import Scaffold
from ...types import NotInGroupCall
from ...types.session import Session


class ResumeStream(Scaffold):
    async def resume_stream(
        self,
        chat_id: Union[int, str],
    ):
        """Resume the paused stream

        This method allow to resume the paused streaming file

        Parameters:
            chat_id (``int`` | ``str``):
                Unique identifier of the target chat.
                Can be a direct id (int) or a username (str)

        Raises:
            NoMtProtoClientSet: In case you try
                to call this method without any MtProto client
            NodeJSNotRunning: In case you try
                to call this method without do
                :meth:`~hikkalls.HikkaLls.start` before
            NotInGroupCallError: In case you try
                to leave a non-joined group call

        Returns:
            ``bool``:
            On success, true is returned if was resumed

        Example:
            .. code-block:: python
                :emphasize-lines: 10-12

                from hikkalls import Client
                from hikkalls import idle
                ...

                app = HikkaLls(client)
                app.start()

                ...  # Call API methods

                app.resume_stream(
                    -1001185324811,
                )

                idle()
        """
        try:
            chat_id = int(chat_id)
        except ValueError:
            chat_id = BridgedClient.chat_id(
                await self._app.resolve_peer(chat_id),
            )
        if self._app is not None:
            if self._wait_until_run is not None:
                solver_id = Session.generate_session_id(24)

                async def internal_sender():
                    if not self._wait_until_run.done():
                        await self._wait_until_run
                    await self._binding.send({
                        'action': 'resume',
                        'chat_id': chat_id,
                        'solver_id': solver_id,
                    })
                active_call = self._call_holder.get_active_call(chat_id)
                asyncio.ensure_future(internal_sender())
                result = await self._wait_result.wait_future_update(
                    solver_id,
                )
                if isinstance(result, NotInGroupCall):
                    raise NotInGroupCallError()
                return active_call.status == 'paused'
            else:
                raise NodeJSNotRunning()
        else:
            raise NoMtProtoClientSet()



from ...ffprobe import FFprobe
from ...media_devices.screen_info import ScreenInfo
from .input_stream import InputStream
from .input_stream import InputVideoStream
from .video_parameters import VideoParameters


class CaptureVideoDesktop(InputStream):
    """Capture video only from Screen

    Attributes:
        ffmpeg_parameters (``str``):
            FFmpeg additional parameters
        stream_audio (:obj:`~hikkalls.types.InputAudioStream()`):
            Input Audio Stream Descriptor
        stream_video (:obj:`~hikkalls.types.InputVideoStream()`):
            Input Video Stream Descriptor
    Parameters:
        screen_info (:obj: `~hikkalls.media_devices.ScreenManager()`):
            The screen video capturing params
        video_parameters (:obj:`~hikkalls.types.VideoParameters()`):
            The video parameters of the stream, can be used also
            :obj:`~hikkalls.types.HighQualityVideo()`,
            :obj:`~hikkalls.types.MediumQualityVideo()` or
            :obj:`~hikkalls.types.LowQualityVideo()`
    """

    def __init__(
        self,
        screen_info: ScreenInfo,
        video_parameters: VideoParameters = VideoParameters(),
    ):
        self._path = screen_info.build_ffmpeg_command(
            video_parameters.frame_rate,
        )
        self.ffmpeg_parameters: str = screen_info.ffmpeg_parameters
        self.raw_headers = None
        super().__init__(
            stream_video=InputVideoStream(
                f'screen://{self._path}',
                video_parameters,
            ),
        )

    @property
    def headers(self):
        return FFprobe.ffmpeg_headers(self.raw_headers)

    async def check_pipe(self):
        pass



from ...ffprobe import FFprobe
from ...media_devices.device_info import DeviceInfo
from .audio_parameters import AudioParameters
from .input_stream import InputAudioStream
from .input_stream import InputStream


class CaptureAudioDevice(InputStream):
    """Capture video from Screen and Audio from device

    Attributes:
        stream_audio (:obj:`~hikkalls.types.InputAudioStream()`):
            Input Audio Stream Descriptor
        stream_video (:obj:`~hikkalls.types.InputVideoStream()`):
            Input Video Stream Descriptor
    Parameters:
        audio_info (:obj: `~hikkalls.media_devices.DeviceInfo()`):
            The audio device capturing params
        audio_parameters (:obj:`~hikkalls.types.AudioParameters()`):
            The audio parameters of the stream, can be used also
            :obj:`~hikkalls.types.HighQualityAudio()`,
            :obj:`~hikkalls.types.MediumQualityAudio()` or
            :obj:`~hikkalls.types.LowQualityAudio()`
    """

    def __init__(
        self,
        audio_info: DeviceInfo,
        audio_parameters: AudioParameters = AudioParameters(),
    ):
        self._audio_path: str = audio_info.build_ffmpeg_command()
        self.ffmpeg_parameters: str = audio_info.ffmpeg_parameters
        self.raw_headers = None
        super().__init__(
            InputAudioStream(
                f'device://{self._audio_path}',
                audio_parameters,
            ),
        )

    @property
    def headers(self):
        return FFprobe.ffmpeg_headers(self.raw_headers)

    async def check_pipe(self):
        pass



import os
from typing import Dict

from setuptools import setup

base_path = os.path.abspath(os.path.dirname(__file__))

about: Dict = {}
with open(
    os.path.join(
        base_path,
        'hikkalls',
        '__version__.py',
    ), encoding='utf-8',
) as f:
    exec(f.read(), about)

setup(
    version=about['__version__'],
    universal=True,
    zip_safe=False,
)
